@@ -5,15 +5,28 @@
 from __future__ import annotations
 
 import json
+import subprocess
+import sys
 import time
 import typing
 import warnings
 from contextlib import contextmanager
 from datetime import datetime
 from pathlib import Path
-from typing import Any, Generic, Iterable, Literal, Optional, Protocol, TypeVar, Union
+from typing import (
+    Any,
+    AsyncIterable,
+    Generic,
+    Iterable,
+    Literal,
+    Optional,
+    Protocol,
+    TypeVar,
+    Union,
+)
 
 import httpx
+import websockets
 from httpx import Timeout
 from tqdm.auto import tqdm
 
@@ -21,6 +34,7 @@ from dyff.schema.adapters import Adapter, create_pipeline
 from dyff.schema.base import DyffBaseModel
 from dyff.schema.dataset import arrow, binary
 from dyff.schema.platform import (
+    Accelerator,
     Artifact,
     ArtifactURL,
     Concern,
@@ -30,9 +44,11 @@ from dyff.schema.platform import (
     Documentation,
     DyffEntity,
     DyffEntityType,
+    Entities,
     Evaluation,
     Family,
     FamilyMemberBase,
+    File,
     InferenceInterface,
     InferenceService,
     InferenceSession,
@@ -41,16 +57,25 @@ from dyff.schema.platform import (
     Measurement,
     Method,
     Model,
+    ModelArtifact,
+    ModelArtifactKind,
+    ModelResources,
+    ModelSource,
+    ModelSourceKinds,
     Module,
+    OCIArtifact,
     Report,
+    Resources,
     SafetyCase,
     Score,
     Status,
     StorageSignedURL,
     UseCase,
+    Volume,
 )
 from dyff.schema.requests import (
     AnalysisCreateRequest,
+    ArtifactCreateRequest,
     ConcernCreateRequest,
     DatasetCreateRequest,
     DocumentationEditRequest,
@@ -66,6 +91,7 @@ from dyff.schema.requests import (
     ModuleCreateRequest,
     ReportCreateRequest,
 )
+from dyff.schema.responses import WorkflowLogEntry, WorkflowLogsResponse
 
 from ._generated.operations._operations import (
     DatasetsOperations as DatasetsOperationsGenerated,
@@ -189,6 +215,31 @@ def _retry_not_found(fn):
                     retries += 1
                 else:
                     raise
+            except httpx.HTTPStatusError as ex:
+                if ex.response.status_code == 404 and retries < len(delays):
+                    time.sleep(delays[retries])
+                    retries += 1
+                else:
+                    raise
+
+    return _impl
+
+
+def _httpx_retry_not_found(fn):
+    def _impl(*args, **kwargs):
+        delays = [1.0, 2.0, 5.0, 10.0, 10.0]
+        retries = 0
+        while True:
+            try:
+                response: httpx.Response = fn(*args, **kwargs)
+                response.raise_for_status()
+                return response
+            except httpx.HTTPStatusError as ex:
+                if ex.response.status_code == 404 and retries < len(delays):
+                    time.sleep(delays[retries])
+                    retries += 1
+                else:
+                    raise
 
     return _impl
 
@@ -254,6 +305,7 @@ _EntityT = TypeVar(
     Method,
     Model,
     Module,
+    OCIArtifact,
     Report,
     SafetyCase,
     UseCase,
@@ -261,6 +313,7 @@ _EntityT = TypeVar(
 _CreateRequestT = TypeVar(
     "_CreateRequestT",
     AnalysisCreateRequest,
+    ArtifactCreateRequest,
     ConcernCreateRequest,
     DatasetCreateRequest,
     EvaluationCreateRequest,
@@ -283,6 +336,7 @@ _CreateResponseT = TypeVar(
     Method,
     Model,
     Module,
+    OCIArtifact,
     Report,
     SafetyCase,
     UseCase,
@@ -379,6 +433,144 @@ class _OpsBase(Generic[_EntityT, _CreateRequestT, _CreateResponseT, _RawOpsT]):
         entity = _retry_not_found(self._raw_ops.create)(request.model_dump(mode="json"))
         return self._response_type.model_validate(entity)
 
+    def _request(self, method: str, route: str, **kwargs) -> httpx.Response:
+        if kwargs.pop("verify", None) is not None:
+            raise ValueError(
+                "overriding 'verify' is not allowed;"
+                " set 'insecure=True' in the client to disable verification"
+            )
+
+        if not route.startswith("/"):
+            route = f"/{route}"
+        headers: dict[str, str] = kwargs.pop("headers", {})
+        headers.setdefault("Authorization", f"Bearer {self._client._api_token}")
+        timeout = kwargs.pop("timeout", self._client.timeout)
+        verify = not self._client.insecure
+
+        response = httpx.request(
+            method,
+            f"{self._client._endpoint}{route}",
+            headers=headers,
+            timeout=timeout,
+            verify=verify,
+            **kwargs,
+        )
+        response.raise_for_status()
+        return response
+
+
+class _NewOpsBase(Generic[_EntityT, _CreateRequestT, _CreateResponseT]):
+    def __init__(
+        self,
+        *,
+        _client: Client,
+        _entity: Entities,
+        _entity_type: type[_EntityT],
+        _request_type: type[_CreateRequestT],
+        _response_type: type[_CreateResponseT],
+    ):
+        self._client = _client
+        self._entity = _entity
+        self._entity_type: type[_EntityT] = _entity_type
+        self._request_type: type[_CreateRequestT] = _request_type
+        self._response_type: type[_CreateResponseT] = _response_type
+
+    @property
+    def _insecure(self) -> bool:
+        return self._client.insecure
+
+    @property
+    def _timeout(self) -> Timeout:
+        return self._client.timeout
+
+    def _route(self, *, id: Optional[str] = None, path: Optional[str] = None) -> str:
+        parts = [Resources.for_kind(self._entity).value]
+        if id is not None:
+            parts.append(id)
+        if path is not None:
+            parts.append(path.removeprefix("/"))
+        return "/".join(parts)
+
+    def get(self, id: str) -> _EntityT:
+        """Get an entity by its .id.
+
+        :param id: The entity ID
+        :return: The entity with the given ID.
+        """
+        response = self._request("GET", self._route(id=id))
+        return self._entity_type.model_validate(response.json())
+
+    def delete(self, id: str) -> Status:
+        """Mark an entity for deletion.
+
+        :param id: The entity ID
+        :return: The resulting status of the entity
+        """
+        response = self._request("PUT", self._route(id=id, path="delete"))
+        return Status.model_validate(response.json())
+
+    def label(self, id: str, labels: dict[str, Optional[str]]) -> None:
+        """Label the specified entity with key-value pairs (stored in the ``.labels``
+        field).
+
+        Providing ``None`` for the value deletes the label. Key-value mappings
+        not given in ``labels`` remain unchanged.
+
+        See :class:`~dyff.schema.platform.Label` for a description of the
+        constraints on label keys and values.
+
+        :param id: The ID of the entity to label.
+        :param labels: The label keys and values.
+        """
+        if not labels:
+            return
+        request = LabelsEditRequest(labels=labels)  # type: ignore
+        self._request(
+            "PATCH",
+            self._route(id=id, path="labels"),
+            json=request.model_dump(mode="json"),
+        )
+
+    def create(self, request: SchemaObject[_CreateRequestT]) -> _CreateResponseT:
+        """Create a new entity.
+
+        .. note::
+            This operation may incur compute costs.
+
+        :param request: The entity create request specification.
+        :return: A full entity spec with its .id and other system properties set
+        """
+        request = _parse_schema_object(self._request_type, request)
+        response = _httpx_retry_not_found(self._request)(
+            "POST", self._route(), json=request.model_dump(mode="json")
+        )
+        return self._response_type.model_validate(response.json())
+
+    def _request(self, method: str, route: str, **kwargs) -> httpx.Response:
+        if kwargs.pop("verify", None) is not None:
+            raise ValueError(
+                "overriding 'verify' is not allowed;"
+                " set 'insecure=True' in the client to disable verification"
+            )
+
+        if not route.startswith("/"):
+            route = f"/{route}"
+        headers: dict[str, str] = kwargs.pop("headers", {})
+        headers.setdefault("Authorization", f"Bearer {self._client._api_token}")
+        timeout = kwargs.pop("timeout", self._client.timeout)
+        verify = not self._client.insecure
+
+        response = httpx.request(
+            method,
+            f"{self._client._endpoint}{route}",
+            headers=headers,
+            timeout=timeout,
+            verify=verify,
+            **kwargs,
+        )
+        response.raise_for_status()
+        return response
+
 
 class _PublishMixin(_OpsProtocol):
     def publish(
@@ -553,6 +745,126 @@ class _LogsMixin(_OpsProtocol):
                         out_stream.write(chunk)
 
 
+class _Artifacts(
+    _NewOpsBase[OCIArtifact, ArtifactCreateRequest, OCIArtifact],
+    _DocumentationMixin,
+    _PublishMixin,
+):
+    """Operations on :class:`~dyff.schema.platform.OCIArtifact` entities.
+
+    .. note::
+
+        Do not instantiate this class. Access it through the
+        ``.artifacts`` attribute of :class:`~dyff.client.Client`.
+    """
+
+    def __init__(
+        self,
+        _client: Client,
+    ):
+        super().__init__(
+            _client=_client,
+            _entity=Entities.Artifact,
+            _entity_type=OCIArtifact,
+            _request_type=ArtifactCreateRequest,
+            _response_type=OCIArtifact,
+        )
+
+    # def query(
+    #     self,
+    #     *,
+    #     query: Optional[QueryT] = None,
+    #     id: Optional[str] = None,
+    #     account: Optional[str] = None,
+    #     status: Optional[str] = None,
+    #     reason: Optional[str] = None,
+    #     labels: Optional[dict[str, str]] = None,
+    # ) -> list[OCIArtifact]:
+    #     """Get all Artifact entities matching a query. The query is a set of equality
+    #     constraints specified as key-value pairs.
+
+    #     :keyword query:
+    #     :keyword id:
+    #     :keyword account:
+    #     :keyword status:
+    #     :keyword reason:
+    #     :keyword labels: Matches entities that are labeled with *all* of the given key-
+    #         value pairs.
+    #     :raises HttpResponseError:
+    #     """
+    #     return [
+    #         OCIArtifact.model_validate(obj)
+    #         for obj in self._raw_ops.query(
+    #             query=_encode_query(query),
+    #             id=id,
+    #             account=account,
+    #             status=status,
+    #             reason=reason,
+    #             labels=_encode_labels(labels),
+    #         )
+    #     ]
+
+    # FIXME: This method should be provided by _DocumentationMixin, but first
+    # we need to migrate all the other types to store documentation as a
+    # member rather than a separate entity.
+
+    # def edit_documentation(
+    #     self, id: str, edit_request: DocumentationEditRequest
+    # ) -> None:
+    #     """Edit the documentation associated with an entity.
+
+    #     :param id: The ID of the entity.
+    #     :param edit_request: Object containing the edits to make.
+    #     """
+    #     self._raw_ops.edit_documentation(id, edit_request.dict())
+
+    def push(self, artifact: OCIArtifact, *, source: str) -> None:
+        """Push an OCI artifact to the Dyff registry.
+
+        The push operation uses the ``skopeo`` command-line tool, which must
+        be installed on the system where the client is running.
+
+        The ``artifact`` must be in ``Created`` status.
+
+        The ``source`` can be any "image name" understood by ``skopeo``.
+
+        :param artifact: An ``OCIArtifact`` entity as returned from ``.create()``.
+        :keyword source: A ``skopeo`` image name referencing the artifact to push.
+        :see: https://github.com/containers/skopeo/blob/main/docs/skopeo.1.md#image-names
+        """
+        registry_username = "unused"
+        registry_password = self._client._api_token
+        registry_credential = f"{registry_username}:{registry_password}"
+        registry_url = self._client._registry_url
+
+        remote_name = f"artifacts/{artifact.id}:latest"
+
+        push_command = [
+            "skopeo",
+            "copy",
+            "--format",
+            "oci",
+            "--dest-creds",
+            registry_credential,
+        ]
+        if self._client._insecure:
+            push_command.append("--dest-tls-verify=false")
+        destination_host = registry_url.removeprefix("https://")
+        destination = f"docker://{destination_host}/{remote_name}"
+        push_command.extend([source, destination])
+
+        subprocess.run(
+            push_command,
+            stdout=sys.stdout,
+            stderr=sys.stderr,
+            check=True,
+        )
+
+    def finalize(self, id: str) -> Status:
+        response = self._request("POST", self._route(id=id, path="finalize"))
+        return Status.model_validate(response.json())
+
+
 class _Datasets(
     _OpsBase[Dataset, DatasetCreateRequest, Dataset, DatasetsOperationsGenerated],
     _ArtifactsMixin,
@@ -1229,6 +1541,59 @@ class _Measurements(
             )
         ]
 
+    def workflow_logs(
+        self, id: str, *, since: str | None = None, limit: int = 1000
+    ) -> WorkflowLogsResponse:
+        """Get workflow infrastructure logs for a Measurement.
+
+        Returns logs from Kubernetes pods running the measurement workflow, including
+        container startup, pod scheduling, and system errors.
+
+        :param id: The ID of the measurement.
+        :param since: Time range like '1h', '30m', '2d'. Optional.
+        :param limit: Maximum number of log lines to return.
+        :return: Workflow logs response with log entries.
+        :raises HttpResponseError:
+        """
+        # Build query parameters
+        params: dict[str, int | str] = {"limit": limit}
+        if since is not None:
+            params["since"] = since
+
+        # Make HTTP request to the measurements workflow logs endpoint
+        # Use the raw client's base URL by reconstructing from endpoint
+        endpoint = self._client._endpoint
+        response = httpx.get(
+            f"{endpoint}/measurements/{id}/logs/workflow",
+            headers={"Authorization": f"Bearer {self._client._api_token}"},
+            params=params,
+            verify=not self._insecure,
+            timeout=self._timeout,
+        )
+        response.raise_for_status()
+        return WorkflowLogsResponse.model_validate(response.json())
+
+    async def stream_workflow_logs(self, id: str) -> AsyncIterable[WorkflowLogEntry]:
+        """Stream workflow logs in real-time via WebSocket.
+
+        :param id: The ID of the measurement.
+        :yield: Individual log entries as they arrive.
+        :raises HttpResponseError:
+        """
+        # Build WebSocket URL
+        base_url = self._client._raw._client._base_url.replace("https://", "").replace(
+            "http://", ""
+        )
+        ws_url = f"wss://{base_url}/v0/measurements/{id}/logs/workflow/stream"
+
+        # Connect to WebSocket with authentication
+        headers = {"Authorization": f"Bearer {self._client._api_token}"}
+
+        async with websockets.connect(ws_url, extra_headers=headers) as websocket:
+            async for message in websocket:
+                log_entry = WorkflowLogEntry.model_validate(json.loads(message))
+                yield log_entry
+
 
 class _Methods(
     _OpsBase[
@@ -1314,6 +1679,58 @@ class _Methods(
             )
         ]
 
+    def workflow_logs(
+        self, id: str, *, since: str | None = None, limit: int = 1000
+    ) -> WorkflowLogsResponse:
+        """Get workflow infrastructure logs for a Method.
+
+        Returns logs from Kubernetes pods running the method workflow, including
+        container startup, pod scheduling, and system errors.
+
+        :param id: The ID of the method.
+        :param since: Time range like '1h', '30m', '2d'. Optional.
+        :param limit: Maximum number of log lines to return.
+        :return: Workflow logs response with log entries.
+        :raises HttpResponseError:
+        """
+        # Build query parameters
+        params: dict[str, int | str] = {"limit": limit}
+        if since is not None:
+            params["since"] = since
+
+        # Make HTTP request to the methods workflow logs endpoint
+        response = httpx.get(
+            f"{self._client._raw._client._base_url}/methods/{id}/logs/workflow",
+            headers={"Authorization": f"Bearer {self._client._api_token}"},
+            params=params,
+            verify=not self._insecure,
+            timeout=self._timeout,
+        )
+        response.raise_for_status()
+        return WorkflowLogsResponse.model_validate(response.json())
+
+    async def stream_workflow_logs(self, id: str) -> AsyncIterable[WorkflowLogEntry]:
+        """Stream workflow logs in real-time via WebSocket.
+
+        :param id: The ID of the method.
+        :yield: Individual log entries as they arrive.
+        :raises HttpResponseError:
+        """
+        # Build WebSocket URL
+        base_url = self._client._raw._client._base_url.replace("https://", "").replace(
+            "http://", ""
+        )
+        ws_url = f"wss://{base_url}/methods/{id}/logs/workflow/stream"
+
+        # Connect to WebSocket with authentication
+        api_token = self._client._api_token
+        headers = {"Authorization": f"Bearer {api_token}"}
+
+        async with websockets.connect(ws_url, extra_headers=headers) as websocket:
+            async for message in websocket:
+                log_entry = WorkflowLogEntry.model_validate(json.loads(message))
+                yield log_entry
+
 
 class _Models(
     _OpsBase[
@@ -1323,6 +1740,7 @@ class _Models(
         ModelsOperationsGenerated,
     ],
     _DocumentationMixin,
+    _LogsMixin,
     _PublishMixin,
 ):
     """Operations on :class:`~dyff.schema.platform.Model` entities.
@@ -1380,6 +1798,186 @@ class _Models(
             )
         ]
 
+    def uplink(self, model_id: str, path: str) -> StorageSignedURL:
+        response = self._request("GET", f"/models/{model_id}/upload/{path}")
+        return StorageSignedURL.model_validate(response.json())
+
+    def finalize(self, model_id: str) -> None:
+        self._request("POST", f"/models/{model_id}/finalize")
+
+    def create_from_volume(
+        self,
+        root_directory: Path | str,
+        *,
+        account: str,
+        name: str,
+        resources: ModelResources,
+        accelerators: Optional[list[Accelerator]] = None,
+        include_symlinks: bool = False,
+    ) -> Model:
+        """Create a Model resource backed by a Volume.
+
+        The Volume will contain all files and directories in the directory tree
+        rooted at ``root_directory``.
+
+        Internally, constructs a ``ModelCreateRequest`` using information
+        obtained from the filesystem, then calls ``create()`` with the
+        constructed request.
+
+        Typical usage::
+
+            model = client.models.create_from_volume(root_directory, ...)
+            client.models.upload_volume(model, root_directory)
+
+        :param root_directory: The root directory of the model data.
+        :keyword account: The account that will own the Model resource.
+        :keyword name: The name of the Model resource.
+        :keyword include_symlinks: If True, the targets of symlinks will be
+            **duplicated** at the path of the symlink. If False, symlinks will
+            be **skipped entirely**. This is because s3-style storage does not
+            have a concept of symlinks.
+        :returns: The complete Model resource.
+        """
+        print(list(root_directory.rglob("*")))
+        root_directory = Path(root_directory)
+        file_paths = [
+            file_path
+            for file_path in root_directory.rglob("*")
+            if file_path.is_file() and (include_symlinks or not file_path.is_symlink())
+        ]
+        upload_paths = [
+            str(Path(file_path).relative_to(root_directory)) for file_path in file_paths
+        ]
+        files = [
+            File(
+                path=upload_path,
+                digest=Digest(
+                    md5=binary.encode(binary.file_digest("md5", str(file_path))),
+                ),
+                # At this point, we've either excluded symlinks or decided to
+                # copy the target. stat() gives info about the target.
+                size=file_path.stat().st_size,
+            )
+            for file_path, upload_path in zip(file_paths, upload_paths)
+        ]
+        request = ModelCreateRequest(
+            name=name,
+            account=account,
+            artifact=ModelArtifact(
+                kind=ModelArtifactKind.Volume,
+                volume=Volume(files=files),
+            ),
+            source=ModelSource(
+                kind=ModelSourceKinds.Upload,
+            ),
+            resources=resources,
+            accelerators=accelerators,
+        )
+        return self.create(request)
+
+    def upload_volume(
+        self,
+        model: Model,
+        root_directory: Path | str,
+    ) -> None:
+        """Uploads the files in a volume for which a Model resource has already been
+        created.
+
+        Typical usage::
+
+            model = client.models.create_from_volume(root_directory, ...)
+            client.models.upload_volume(model, root_directory)
+
+        :param model: The Model resource to be backed by the volume.
+        :param root_directory: The root directory of the model data.
+        """
+        if model.artifact.kind != ModelArtifactKind.Volume:
+            raise ValueError(f"expected .artifact.kind == {ModelArtifactKind.Volume}")
+        if model.artifact.volume is None:
+            raise ValueError("expected .artifact.volume")
+
+        if any(file.digest.md5 is None for file in model.artifact.volume.files):
+            raise ValueError("file.digest.md5 must be set for all files")
+        for file in model.artifact.volume.files:
+            assert file.digest.md5 is not None
+            file_path = Path(root_directory) / file.path
+            put_url: StorageSignedURL = _retry_not_found(self.uplink)(
+                model.id, file.path
+            )
+            if put_url.method != "PUT":
+                raise ValueError(f"expected a PUT URL; got {put_url.method}")
+
+            file_size = file_path.stat().st_size
+            with open(file_path, "rb") as fin:
+                with _file_upload_progress_bar(
+                    fin, total=file_size, desc=file.path
+                ) as content:
+                    headers = {
+                        "content-md5": file.digest.md5,
+                    }
+                    headers.update(put_url.headers)
+                    response = httpx.put(
+                        put_url.url,
+                        content=content,
+                        headers=headers,
+                        verify=not self._insecure,
+                        timeout=self._timeout,
+                    )
+                    response.raise_for_status()
+        _retry_not_found(self.finalize)(model.id)
+
+    def workflow_logs(
+        self, id: str, *, since: str | None = None, limit: int = 1000
+    ) -> WorkflowLogsResponse:
+        """Get workflow infrastructure logs for a Model.
+
+        Returns logs from Kubernetes pods running the model workflow, including
+        container startup, pod scheduling, and system errors.
+
+        :param id: The ID of the model.
+        :param since: Time range like '1h', '30m', '2d'. Optional.
+        :param limit: Maximum number of log lines to return.
+        :return: Workflow logs response with log entries.
+        :raises HttpResponseError:
+        """
+        # Build query parameters
+        params: dict[str, int | str] = {"limit": limit}
+        if since is not None:
+            params["since"] = since
+
+        # Make HTTP request to the models workflow logs endpoint
+        response = httpx.get(
+            f"{self._client._raw._client._base_url}/models/{id}/logs/workflow",
+            headers={"Authorization": f"Bearer {self._client._api_token}"},
+            params=params,
+            verify=not self._insecure,
+            timeout=self._timeout,
+        )
+        response.raise_for_status()
+        return WorkflowLogsResponse.model_validate(response.json())
+
+    async def stream_workflow_logs(self, id: str) -> AsyncIterable[WorkflowLogEntry]:
+        """Stream workflow logs in real-time via WebSocket.
+
+        :param id: The ID of the model.
+        :yield: Individual log entries as they arrive.
+        :raises HttpResponseError:
+        """
+        # Build WebSocket URL
+        base_url = self._client._raw._client._base_url.replace("https://", "").replace(
+            "http://", ""
+        )
+        ws_url = f"wss://{base_url}/models/{id}/logs/workflow/stream"
+
+        # Connect to WebSocket with authentication
+        api_token = self._client._api_token
+        headers = {"Authorization": f"Bearer {api_token}"}
+
+        async with websockets.connect(ws_url, extra_headers=headers) as websocket:
+            async for message in websocket:
+                log_entry = WorkflowLogEntry.model_validate(json.loads(message))
+                yield log_entry
+
 
 class _Modules(
     _OpsBase[
@@ -1769,6 +2367,79 @@ class _SafetyCases(
             )
         ]
 
+    def workflow_logs(
+        self, id: str, *, since: str | None = None, limit: int = 1000
+    ) -> WorkflowLogsResponse:
+        """Get workflow infrastructure logs for a SafetyCase.
+
+        Returns logs from Kubernetes pods running the safety case workflow, including
+        container startup, pod scheduling, and system errors.
+
+        :param id: The ID of the safety case.
+        :param since: Time range like '1h', '30m', '2d'. Optional.
+        :param limit: Maximum number of log lines to return.
+        :return: Workflow logs response with log entries.
+        :raises HttpResponseError:
+        """
+        # Build query parameters
+        params: dict[str, int | str] = {"limit": limit}
+        if since is not None:
+            params["since"] = since
+
+        # Make HTTP request to the safetycases workflow logs endpoint
+        response = httpx.get(
+            f"{self._client._raw._client._base_url}/safetycases/{id}/logs/workflow",
+            headers={"Authorization": f"Bearer {self._client._api_token}"},
+            params=params,
+            verify=not self._insecure,
+            timeout=self._timeout,
+        )
+        response.raise_for_status()
+        return WorkflowLogsResponse.model_validate(response.json())
+
+    async def stream_workflow_logs(self, id: str) -> AsyncIterable[WorkflowLogEntry]:
+        """Stream workflow logs in real-time via WebSocket.
+
+        :param id: The ID of the safety case.
+        :yield: Individual log entries as they arrive.
+        :raises HttpResponseError:
+        """
+        # Build WebSocket URL
+        base_url = self._client._raw._client._base_url.replace("https://", "").replace(
+            "http://", ""
+        )
+        ws_url = f"wss://{base_url}/safetycases/{id}/logs/workflow/stream"
+
+        # Connect to WebSocket with authentication
+        api_token = self._client._api_token
+        headers = {"Authorization": f"Bearer {api_token}"}
+
+        async with websockets.connect(ws_url, extra_headers=headers) as websocket:
+            async for message in websocket:
+                log_entry = WorkflowLogEntry.model_validate(json.loads(message))
+                yield log_entry
+
+
+class _Audits(_LogsMixin):
+    """Operations on :class:`~dyff.schema.platform.Audit` entities.
+
+    .. note::
+
+        Do not instantiate this class. Access it through the
+        ``.audits`` attribute of :class:`~dyff.client.Client`.
+    """
+
+    def __init__(self, _client: Client):
+        self._client = _client
+
+    @property
+    def _insecure(self) -> bool:
+        return self._client._insecure
+
+    @property
+    def _timeout(self) -> Timeout:
+        return self._client._timeout
+
 
 class _UseCases(
     _OpsBase[UseCase, ConcernCreateRequest, UseCase, UsecasesOperationsGenerated],
@@ -1845,6 +2516,8 @@ class _UseCases(
 
 
 __all__ = [
+    "_Artifacts",
+    "_Audits",
     "_Datasets",
     "_Evaluations",
     "_Families",
